/***********************************************************************/
/* MSP430                                                              */
/* Flash memory access routines - Patrick 'OldBug' Dohmen, 2004        */
/*                                                                     */
/* IMPORTANT NOTICE:                                                   */
/* -----------------                                                   */
/*                                                                     */
/*  - before calling one of these functions, you WILL HAVE TO disable  */
/*    your global intrerrupts and save your IEx-registers!             */
/*    Example:                                                         */
/*                                                                     */
/*    void                                                             */
/*    foo(void)                                                        */
/*    {                                                                */
/*        unsigned char tmpbuf;                                        */
/*                                                                     */
/*        dint();                                                      */
/*        tmpbuf = IE1;                                                */
/*        IE1    = 0x00;                                               */
/*                                                                     */
/*        Flash_wb(0x01, 0x55);                                        */
/*                                                                     */
/*        IE1    = tmpbuf;                                             */
/*        eint();                                                      */
/*    }                                                                */
/*                                                                     */
/*  - TODO: Fix some awfull things in here, this is very undocumented  */
/*          and dirty code! :-(                                        */
/*                                                                     */
/***********************************************************************/

#include "flash_access.h"    /* function prototypes */

/*
 * Flash_clr: erases 1 segment of flash memory
 */

void Flash_clrb(unsigned int *Data_ptr)
{
    FCTL2 = 0x0A549;
    FCTL3 = 0x0A500;      /* ERASE of one segment enabled  */
    FCTL1 = 0x0A502;      /* unlock the Flash Lock = 0     */

    *Data_ptr = 0;

    /* erase Flash segment */
    FCTL1 = 0x0A500;
    FCTL3 = 0x0A510;
}

/*
 * Flash_wb: programs 1 byte (8 bit) into the flash memory
 */

void Flash_wb(unsigned int ptr, unsigned char byte)
{
    unsigned char Flash[0x80];
    unsigned int  z;
    unsigned int  *Flash_ptr;

    Flash_ptr = (unsigned char *) FLASH_SEGMENT;

    /* Save flash content: */
    for (z = 0; z < 0x80; z++)
    {
        Flash[z] = *Flash_ptr++;
    }

    /* Modify adressed byte: */
    Flash[ptr] = byte;

    Flash_ptr = (unsigned int *) FLASH_SEGMENT;
    Flash_clrb(Flash_ptr);
    Flash_ptr = (unsigned int *) FLASH_SEGMENT;

    FCTL2 = 0x0A549;
    FCTL3 = 0x0A500;      /* ERASE of one segment enabled  */
    FCTL1 = 0x0A540;      /* unlock the Flash Lock = 0     */

    /* Store modified content in flash memory: */
    for (z = 0; z < 0x80; z++)
    {
        *Flash_ptr++ = Flash[z];
    }

    FCTL1 = 0x0A500;
    FCTL3 = 0x0A510;
}


/*
 * Flash_ww: programs 1 word (16 bit) into the flash memory
 */
 
void Flash_ww(unsigned int ptr, unsigned int word)
{
    Flash_wb(ptr++, (char) ((word>>8) & 0xFF));
    Flash_wb(ptr, (char) ((word) & 0xFF));
}

/*
 * Flash_wb: programs 1 long( 32 bit) into the flash memory
 */
 
void Flash_wl(unsigned int ptr, unsigned long lword)
{
    Flash_wb(ptr++, (char) ((lword>>24) & 0xFF));
    Flash_wb(ptr++, (char) ((lword>>16) & 0xFF));
    Flash_wb(ptr++, (char) ((lword>>8) & 0xFF));
    Flash_wb(ptr, (char) ((lword) & 0xFF));
}

/*
 * Flash_rb: reads 1 byte (8 bit) from flash memory
 */
 
unsigned char Flash_rb(unsigned int ptr)
{
    unsigned int *Flash_ptr;
    unsigned char value = 0;

    Flash_ptr = (unsigned int *) FLASH_SEGMENT;
    Flash_ptr += ptr;

    value = *Flash_ptr;

    return(value);
}

/*
 * Flash_rw: programs 1 word (16 bit) read from flash memory
 */
 
unsigned int Flash_rw(unsigned int ptr)
{
    unsigned int *Flash_ptr;
    unsigned int value = 0;

    Flash_ptr = (unsigned int *) FLASH_SEGMENT;
    Flash_ptr += ptr;

    value = (unsigned char) *Flash_ptr++;
    value = value << 8;
    value = value + (unsigned char) *Flash_ptr;

    return(value);
}

/*
 * Flash_rl: reads 1 long (32 bit) from flash memory
 */
 
unsigned long Flash_rl(unsigned int ptr)
{
    unsigned int *Flash_ptr;
    unsigned long value = 0;
    
    Flash_ptr = (unsigned int*) FLASH_SEGMENT;
    Flash_ptr += ptr;

    value = (unsigned char) *Flash_ptr++;
    value = value << 8;
    value = value + (unsigned char) *Flash_ptr++;
    value = value << 8;
    value = value + (unsigned char) *Flash_ptr++;
    value = value << 8;
    value = value + (unsigned char) *Flash_ptr;

    return(value);
}
